@@ -1,2 +1,2 @@
 /* This date time is from the last non-merge commit to Hamlib. */
-#define HAMLIBDATETIME "Thu Dec 02 23:46:51 2021 +0000 SHA=5f8c4c"
+#define HAMLIBDATETIME "Thu Dec 08 15:11:46 2022 +0000 SHA=75fbd0"
