@@ -19,25 +19,48 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
+#include <hamlib/config.h>
 
 #include <stdio.h>
 #include <string.h>
-#include <stdlib.h>
 
 #include <hamlib/rig.h>
 #include "misc.h"
 
 #include "sprintflst.h"
 #include "rigctl_parse.h"
+#include "../rigs/icom/icom.h"
 
 void range_print(FILE *fout, const struct freq_range_list range_list[], int rx);
 int range_sanity_check(const struct freq_range_list range_list[], int rx);
 int ts_sanity_check(const struct tuning_step_list tuning_step[]);
 static void dump_chan_caps(const channel_cap_t *chan, FILE *fout);
 
+struct rig_type_s
+{
+    int type;
+    char *description;
+};
+
+struct rig_type_s rig_type[] =
+{
+    {RIG_TYPE_OTHER, "Other"},
+    {RIG_FLAG_RECEIVER, "Receiver"},
+    {RIG_FLAG_TRANSMITTER, "Transmitter"},
+    {RIG_FLAG_SCANNER, "Scanner"},
+    {RIG_FLAG_MOBILE, "Mobile"},
+    {RIG_FLAG_HANDHELD, "Handheld"},
+    {RIG_FLAG_COMPUTER, "Computer"},
+    {RIG_FLAG_TRANSCEIVER, "Transceiver"},
+    {RIG_FLAG_TRUNKING, "Trunking scanner"},
+    {RIG_FLAG_APRS, "APRS"},
+    {RIG_FLAG_TNC, "TNC"},
+    {RIG_FLAG_DXCLUSTER, "DxCluster"},
+    {RIG_FLAG_DXCLUSTER, "DxCluster"},
+    {RIG_FLAG_TUNER, "Tuner"},
+    {-1, "?\n"}
+};
+
 static int print_ext(RIG *rig, const struct confparams *cfp, rig_ptr_t ptr)
 {
     return print_ext_param(cfp, ptr);
@@ -53,10 +76,14 @@ int dumpcaps(RIG *rig, FILE *fout)
     int can_esplit, can_echannel;
     char freqbuf[20];
     int backend_warnings = 0;
-    static char prntbuf[1024];  /* a malloc would be better.. */
+    char warnbuf[4096];
+    char prntbuf[1024];  /* a malloc would be better.. */
     char *label1, *label2, *label3, *label4, *label5;
     char *labelrx1; // , *labelrx2, *labelrx3, *labelrx4, *labelrx5;
 
+    warnbuf[0] = 0;
+    prntbuf[0] = 0;
+
     if (!rig || !rig->caps)
     {
         return -RIG_EINVAL;
@@ -72,50 +99,22 @@ int dumpcaps(RIG *rig, FILE *fout)
     fprintf(fout, "Backend status:\t%s\n", rig_strstatus(caps->status));
     fprintf(fout, "Rig type:\t");
 
-    switch (caps->rig_type & RIG_TYPE_MASK)
+    char *unknown = "Unknown";
+
+    for (i = 0; rig_type[i].type != -1; ++i)
     {
-    case RIG_TYPE_TRANSCEIVER:
-        fprintf(fout, "Transceiver\n");
-        break;
+        if ((rig_type[i].type & caps->rig_type) == rig_type[i].type)
+        {
+            fprintf(fout, "%s ", rig_type[i].description);
+            unknown = "";
+        }
+    }
 
-    case RIG_TYPE_HANDHELD:
-        fprintf(fout, "Handheld\n");
-        break;
+    fprintf(fout, "%s\n", unknown);
 
-    case RIG_TYPE_MOBILE:
-        fprintf(fout, "Mobile\n");
-        break;
-
-    case RIG_TYPE_RECEIVER:
-        fprintf(fout, "Receiver\n");
-        break;
-
-    case RIG_TYPE_PCRECEIVER:
-        fprintf(fout, "PC Receiver\n");
-        break;
-
-    case RIG_TYPE_SCANNER:
-        fprintf(fout, "Scanner\n");
-        break;
-
-    case RIG_TYPE_TRUNKSCANNER:
-        fprintf(fout, "Trunking scanner\n");
-        break;
-
-    case RIG_TYPE_COMPUTER:
-        fprintf(fout, "Computer\n");
-        break;
-
-    case RIG_TYPE_TUNER:
-        fprintf(fout, "Tuner\n");
-        break;
-
-    case RIG_TYPE_OTHER:
-        fprintf(fout, "Other\n");
-        break;
-
-    default:
-        fprintf(fout, "Unknown\n");
+    if (strlen(unknown) > 0)
+    {
+        strcat(warnbuf, " RIG_TYPE");
         backend_warnings++;
     }
 
@@ -149,6 +148,7 @@ int dumpcaps(RIG *rig, FILE *fout)
 
     default:
         fprintf(fout, "Unknown\n");
+        strcat(warnbuf, " PTT_TYPE");
         backend_warnings++;
     }
 
@@ -182,6 +182,7 @@ int dumpcaps(RIG *rig, FILE *fout)
 
     default:
         fprintf(fout, "Unknown\n");
+        strcat(warnbuf, " DCD_TYPE");
         backend_warnings++;
     }
 
@@ -232,6 +233,7 @@ int dumpcaps(RIG *rig, FILE *fout)
 
     default:
         fprintf(fout, "Unknown\n");
+        strcat(warnbuf, " PORT_TYPE");
         backend_warnings++;
     }
 
@@ -247,8 +249,8 @@ int dumpcaps(RIG *rig, FILE *fout)
             caps->targetable_vfo ? "Y" : "N");
 
     fprintf(fout,
-            "Has transceive: %s\n",
-            caps->transceive ? "Y" : "N");
+            "Has async data support: %s\n",
+            caps->async_data_supported ? "Y" : "N");
 
     fprintf(fout, "Announce: 0x%x\n", caps->announces);
     fprintf(fout,
@@ -293,13 +295,38 @@ int dumpcaps(RIG *rig, FILE *fout)
 
     fprintf(fout, "\n");
 
-    fprintf(fout, "AGC levels:");
 
-    for (i = 0; i < HAMLIB_MAX_AGC_LEVELS && i < caps->agc_level_count; i++)
+#if 0
+    fprintf(fout, "AGC levels:");
+    const struct icom_priv_caps *priv_caps =
+        (const struct icom_priv_caps *) rig->caps->priv;
+
+    if (priv_caps && RIG_BACKEND_NUM(rig->caps->rig_model) == RIG_ICOM
+            && priv_caps->agc_levels_present)
     {
-        fprintf(fout, " %d=%s", caps->agc_levels[i],
-                rig_stragclevel(caps->agc_levels[i]));
+        for (i = 0; i <= RIG_AGC_LAST && priv_caps->agc_levels[i].level != RIG_AGC_LAST
+                && priv_caps->agc_levels[i].icom_level >= 0; i++)
+        {
+            fprintf(fout, " %d=%s", priv_caps->agc_levels[i].level,
+                    rig_stragclevel(priv_caps->agc_levels[i].level));
+        }
     }
+    else
+    {
+        for (i = 0; i < HAMLIB_MAX_AGC_LEVELS && i < caps->agc_level_count; i++)
+        {
+            fprintf(fout, " %d=%s", caps->agc_levels[i],
+                    rig_stragclevel(caps->agc_levels[i]));
+        }
+    }
+
+#else
+    //fprintf(fout, "\n");
+    fprintf(fout, "AGC levels: ");
+    char buf[1024];
+    rig_sprintf_agc_levels(rig, buf, (int)sizeof(buf));
+    fprintf(fout, "%s", buf);
+#endif
 
     if (i == 0)
     {
@@ -370,6 +397,7 @@ int dumpcaps(RIG *rig, FILE *fout)
 
         fprintf(fout,
                 "Warning--backend has get RAWSTR, but not calibration data\n");
+        strcat(warnbuf, " RAWSTR_level");
         backend_warnings++;
     }
 
@@ -382,6 +410,7 @@ int dumpcaps(RIG *rig, FILE *fout)
 
         //fprintf(fout, "Warning--backend can set readonly levels=0x%0llx\n", caps->has_set_level & RIG_LEVEL_READONLY_LIST);
         fprintf(fout, "Warning--backend can set readonly levels\n");
+        strcat(warnbuf, " READONLY_LEVEL");
         backend_warnings++;
     }
 
@@ -399,6 +428,7 @@ int dumpcaps(RIG *rig, FILE *fout)
     if (caps->has_set_parm & RIG_PARM_READONLY_LIST)
     {
         fprintf(fout, "Warning--backend can set readonly parms!\n");
+        strcat(warnbuf, " READONLY_PARM");
         backend_warnings++;
     }
 
@@ -413,6 +443,7 @@ int dumpcaps(RIG *rig, FILE *fout)
     else
     {
         strcpy(prntbuf, "None. This backend might be bogus!\n");
+        strcat(warnbuf, " MODE_LIST");
         backend_warnings++;
     }
 
@@ -425,6 +456,7 @@ int dumpcaps(RIG *rig, FILE *fout)
     else
     {
         strcpy(prntbuf, "None. This backend might be bogus!\n");
+        strcat(warnbuf, " VFO_LIST");
         backend_warnings++;
     }
 
@@ -517,6 +549,7 @@ int dumpcaps(RIG *rig, FILE *fout)
 
     if (status)
     {
+        strcat(warnbuf, " TX#1");
         backend_warnings++;
     }
 
@@ -528,6 +561,7 @@ int dumpcaps(RIG *rig, FILE *fout)
 
     if (status)
     {
+        strcat(warnbuf, " RX#1");
         backend_warnings++;
     }
 
@@ -539,6 +573,7 @@ int dumpcaps(RIG *rig, FILE *fout)
 
     if (status)
     {
+        strcat(warnbuf, " TX#2");
         backend_warnings++;
     }
 
@@ -550,6 +585,7 @@ int dumpcaps(RIG *rig, FILE *fout)
 
     if (status)
     {
+        strcat(warnbuf, " RX#2");
         backend_warnings++;
     }
 
@@ -561,6 +597,7 @@ int dumpcaps(RIG *rig, FILE *fout)
 
     if (status)
     {
+        strcat(warnbuf, " TX#3");
         backend_warnings++;
     }
 
@@ -572,6 +609,7 @@ int dumpcaps(RIG *rig, FILE *fout)
 
     if (status)
     {
+        strcat(warnbuf, " RX#3");
         backend_warnings++;
     }
 
@@ -583,6 +621,7 @@ int dumpcaps(RIG *rig, FILE *fout)
 
     if (status)
     {
+        strcat(warnbuf, " TX#4");
         backend_warnings++;
     }
 
@@ -594,6 +633,7 @@ int dumpcaps(RIG *rig, FILE *fout)
 
     if (status)
     {
+        strcat(warnbuf, " RX#4");
         backend_warnings++;
     }
 
@@ -605,6 +645,7 @@ int dumpcaps(RIG *rig, FILE *fout)
 
     if (status)
     {
+        strcat(warnbuf, " TX#5");
         backend_warnings++;
     }
 
@@ -616,6 +657,7 @@ int dumpcaps(RIG *rig, FILE *fout)
 
     if (status)
     {
+        strcat(warnbuf, " RX#5");
         backend_warnings++;
     }
 
@@ -639,6 +681,7 @@ int dumpcaps(RIG *rig, FILE *fout)
     if (i == 0)
     {
         fprintf(fout, " None! This backend might be bogus!");
+        strcat(warnbuf, " TUNING_STEPS");
         backend_warnings++;
     }
 
@@ -648,6 +691,7 @@ int dumpcaps(RIG *rig, FILE *fout)
 
     if (status)
     {
+        strcat(warnbuf, " TUNING_SANE");
         backend_warnings++;
     }
 
@@ -671,6 +715,7 @@ int dumpcaps(RIG *rig, FILE *fout)
     if (i == 0)
     {
         fprintf(fout, " None. This backend might be bogus!");
+        strcat(warnbuf, " FILTERS");
         backend_warnings++;
     }
 
@@ -684,7 +729,7 @@ int dumpcaps(RIG *rig, FILE *fout)
 
         if (pbnorm == 0)
         {
-//            continue;
+            continue;
         }
 
         sprintf_freq(freqbuf, sizeof(freqbuf), pbnorm);
@@ -889,7 +934,8 @@ int dumpcaps(RIG *rig, FILE *fout)
     fprintf(fout, "Can get power2mW:\t%c\n", caps->power2mW != NULL ? 'Y' : 'N');
     fprintf(fout, "Can get mW2power:\t%c\n", caps->mW2power != NULL ? 'Y' : 'N');
 
-    fprintf(fout, "\nOverall backend warnings: %d\n", backend_warnings);
+    fprintf(fout, "\nOverall backend warnings: %d %c %s\n", backend_warnings,
+            warnbuf[0] != 0 ? '=' : ' ', warnbuf);
 
     return backend_warnings;
 }
@@ -1003,7 +1049,8 @@ int range_sanity_check(const struct freq_range_list range_list[], int rx)
         }
         else
         {
-            if (!(range_list[i].low_power > 0 && range_list[i].high_power > 0))
+            if (!(range_list[i].low_power >= RIG_FREQ_NONE
+                    && range_list[i].high_power >= RIG_FREQ_NONE))
             {
                 return -3;
             }
